package org.ninjasoft.rovr.autopilots;

import org.ninjasoft.rovr.map.Mapper;
import org.ninjasoft.rovr.net.Client;

public class CorridorAutopilot extends BaseAutopilot {
	
	protected int currentDirection = Client.COMMAND_DOWN;
	protected int state;
	
	public CorridorAutopilot(AutopilotStatus status, Client client, Mapper mapper) {
		super(status, client, mapper);
		moduleName = "Corridor Autopilot";
		moduleAuthor = "Brian Enigma";
	}
	
	protected void initialize() throws Exception {
		client.sendCommand(Client.COMMAND_STOP);
		client.setLean(0, 0);
		client.setMoveFactor(15);
	}

	protected int doState(int state) throws Exception {
		int responseCode = -1;
		System.out.println("Current state is: " + state);
		switch(state) {
		case 0: // Go forward until move or timeout
			client.sendCommand(currentDirection);
			mapper.direction = currentDirection;
			responseCode = waitForCodeOrTimeout(5);
			if (responseCode == Client.RESPONSE_MOVED) // If moved, keep going forward
				return 0;
			else if ((responseCode == Client.RESPONSE_HIT_LEFT_RIGHT) || (responseCode == Client.RESPONSE_HIT_UP_DOWN))
				return 1; // Hit a wall, go to next state
			else if (responseCode == -1) // Timeout
				return 1;	// Timeout... maybe we're already against a wall?
			break;
		case 1: // Go forward one more time, just to ensure we are right up against the wall
			client.sendCommand(currentDirection);
			responseCode = waitForCodeOrTimeout(10);
			if (responseCode == -1)
				return 2;	// Didn't register wall hit.  We're right against it
			else
				return 1;	// Got some response code, keep moving until we're close enough to not register hit
		case 2: // Make a right turn and try for a corridor down that direction
			currentDirection = turnRight(currentDirection);
			mapper.direction = currentDirection;
			client.sendCommand(currentDirection);
			responseCode = waitForCodeOrTimeout(5);
			if (responseCode == Client.RESPONSE_MOVED) // If we moved, make this the new forward
				return 0;
			else
				return 3;
		case 3: // Return to center, make left turn, then try for a corridor in that direction
			currentDirection = turnLeft(currentDirection);
			currentDirection = turnLeft(currentDirection);
			mapper.direction = currentDirection;
			client.sendCommand(currentDirection);
			responseCode = waitForCodeOrTimeout(5);
			if (responseCode == Client.RESPONSE_MOVED) {// If we moved, make this the new forward
				return 0;
			} else {
				// Must be in a dead end?
				// TODO: better algorithm
				currentDirection = turnLeft(currentDirection);
				return 0;
			}
		}
		// default to dropping back to state 0
		return 0;
	}

}
