package org.ninjasoft.rovr.util;
import java.io.*;
import java.util.jar.*;

public class Launcher {
	public static void launchMaze(boolean patchWallElasticity, boolean patchFriction) throws IOException {
		String jarFile;
		if (patchWallElasticity || patchFriction) {
			patchMaze(patchWallElasticity, patchFriction);
			jarFile = "maze-patched.jar";
		} else {
			jarFile = "maze.jar";
		}
		Runtime.getRuntime().exec(new String[] {"java", "-jar", jarFile});
	}
	
	/**
	 * Patch the maze jar file.   
	 * @param patchWallElasticity apply patch such that walls are no longer "bouncy."  When you hit a wall, it absorbs all
	 *                            shock and you never get thrown back a pixel or two.
	 * @param patchFriction apply patch for altering the coefficient of friction of the ball against the ground.  
	 *                      This doesn't do anything useful, except from preventing you from stopping easily!
	 * @throws IOException
	 */
	public static void patchMaze(boolean patchWallElasticity, boolean patchFriction) throws IOException {
		byte block[] = new byte[1024];
		int len;
		File f = new File("maze-patched.jar");
		if (f.exists())
			f.delete();
		JarInputStream inJar = new JarInputStream(new FileInputStream("maze.jar"));
		JarOutputStream outJar = new JarOutputStream(new FileOutputStream("maze-patched.jar"), inJar.getManifest());
		JarEntry entry;
		while ((entry = inJar.getNextJarEntry()) != null) {
			System.out.println(entry.getName());
			if (entry.getName().equals("tilt/game/h.class")) {	
				// Patch h.class
				FileOutputStream out = new FileOutputStream("h.class.original");
				FileInputStream in;
				while ((len = inJar.read(block)) > 0) {
					out.write(block);
				}
				out.close();
				patchClass(patchWallElasticity, patchFriction);
				entry.setCompressedSize(-1);
				in = new FileInputStream("h.class.patched");
				outJar.putNextEntry(entry);
				while ((len = in.read(block)) > 0) {
					outJar.write(block, 0, len);
				}
				in.close();
			} else {	
				// Pass all other files through uscathed
				outJar.putNextEntry(entry);
				while ((len = inJar.read(block)) > 0) {
					outJar.write(block, 0, len);
				}
			}
		}
		inJar.close();
		outJar.close();
	}
	
	private static void patchClass(boolean patchWallElasticity, boolean patchFriction) throws IOException {
		int floatData[] = new int[4];
		int val;
		FileInputStream in = new FileInputStream("h.class.original");
		FileOutputStream out = new FileOutputStream("h.class.patched");
		for (int i=0; i<0x69;i++)
			out.write(in.read());
		floatData[0] = in.read();
		floatData[1] = in.read();
		floatData[2] = in.read();
		floatData[3] = in.read();
		if (    ((floatData[0] != 0x3f) && (floatData[0] != 0x00))
		     || ((floatData[1] != 0x33) && (floatData[1] != 0x00))
		     || ((floatData[2] != 0x33) && (floatData[2] != 0x00))
		     || ((floatData[3] != 0x33) && (floatData[3] != 0x00))) {
			System.err.println("Could not find elasticity byte to patch!");
			throw new IOException();
		}
		if (patchWallElasticity) {
			out.write(0x00);
			out.write(0x00);
			out.write(0x00);
			out.write(0x00);
		}else{
			out.write(0x3f);
			out.write(0x33);
			out.write(0x33);
			out.write(0x33);
		}
		out.write(in.read());
		floatData[0] = in.read();
		floatData[1] = in.read();
		floatData[2] = in.read();
		floatData[3] = in.read();
		if (    ((floatData[0] != 0x3f) && (floatData[0] != 0x00))
			 || ((floatData[1] != 0x7e) && (floatData[1] != 0x00))
			 || ((floatData[2] != 0xb8) && (floatData[2] != 0x00))
			 || ((floatData[3] != 0x52) && (floatData[3] != 0x00))) {
				System.err.println("Could not find friction byte to patch!");
				throw new IOException();
		}
		if (patchFriction) {
			out.write(0x3f);
			out.write(0x80);
			out.write(0x00);
			out.write(0x00);
		}else{
			out.write(0x3f);
			out.write(0x7e);
			out.write(0xb8);
			out.write(0x52);
		}
		while ((val = in.read()) != -1)
			out.write(val);
		in.close();
		out.close();
	}

	public static void main(String argv[]) {
		try{
			launchMaze(true, true);
		}catch(Throwable t) {
			t.printStackTrace();
		}
	}
}
